from PyLTSpice import SimRunner, SpiceEditor, RawRead
import numpy as np
import matplotlib.pyplot as plt

'''
Este codigo funciona sobre modelo.net

Para conseguir este archivo hay que ir al .asc con
el mismo nombre y ir View > SPICE Netlist. Esto
mostrara el codigo que se quiere. Se copia el codigo
y se pega en un file con extenion .net

'''

__author__ = 'Oscar Suescun'

def simular(tensionCap, Rtotal, Lbobina, Ctotal, lts_path, tSim = 100e-3, toff_sw = 1e-3, ton_mos = 1.5e-3):

    '''
    Funcion principal de simulacion.

    Entradas Obligatorias:
        - tensionCap -> Tension del capacitor
        - Rtotal -> Resistencia total en el circuito (Rextra + Rbobina)
        - Lbobina -> Inductancia de la bobina
        - Ctotal -> capacidad del banco de capacitores
        - lts_path -> directorio donde tienes el .exe de LTSpice
    
    Entradas Opcionales:
        - tSim -> tiempo total de simulacion (de normal 100ms)
        - toff_sw -> tiempo que tarda la fuente en 
                     desconectarse del capacitor (de normal 1ms)
        - ton_mos -> tiempo que tarda el mosfet en abrir el gate 
                     del mosfet de disparo (de normal 1.5ms)

    Salida
        resultados = {
            'tiempo' : vector con el tiempo de simulacion
            'vCap' : Vector con la tension entre Capacitor -> GND
            'vBob' : Vector de tension atraves de la bobina
            'iBob' : Vector con la corriente atraves de Bobina
        }

    '''

    modeloSim = "src/simulador/modelo_transitorio.asc"
    outDir = 'src/simulador'

    ## Hago la simulacion

    simulador = SimRunner(output_folder=outDir, simulator=lts_path)

    simulador.create_netlist(modeloSim)
    netlist = SpiceEditor(modeloSim.replace('.asc', '.net'))

    netlist.set_parameters(V = tensionCap, 
                           R = Rtotal, 
                           L = Lbobina, 
                           C = Ctotal, 
                           tSim = tSim, 
                           toff_sw = toff_sw, 
                           ton_mos = ton_mos)
    
    simulador.run(netlist)

    raw_path, log_path = next(iter(simulador))  # Obtener el único resultado
    raw = RawRead(raw_path)

    try:
        return {
            'tiempo': np.array(raw.get_trace('time')),
            'vCap': np.array(raw.get_trace('V(condensador)')),
            'vBob': np.array(raw.get_trace('V(v1)')) - np.array(raw.get_trace('V(v2)')),
            'iBob': np.array(raw.get_trace('I(L1)')),
            'iMos' : np.array(raw.get_trace('Id(M1)'))
        }
    except KeyError as e:
        print(f'Error al leer señal del archivo .raw: {e}')
        return None
    
def dibujar(resultado):
    plt.figure()
    plt.plot(resultado['tiempo']*1e3, resultado['vCap'], label='Tension Cap [V]')
    plt.plot(resultado['tiempo']*1e3, resultado['vBob'], label='Tension Bobina [V]')
    plt.plot(resultado['tiempo']*1e3, resultado['iBob'], label='Corriente Bobina [A]')
    plt.grid()
    plt.legend()
    plt.show()
        
        
if __name__ == '__main__':

    lts_path = "C:/Users/osuescuneli/AppData/Local/Programs/ADI/LTspice/LTspice.exe"

    Tension = 150
    Resistencia = 4
    Inductancia = 100e-6
    Capacitancia = 500e-6

    tiempoSimulacion = 15e-3
    tiempoSwitch = 25e-6
    tiempoMos = 50e-6


    resultado = simular(Tension, 
                        Resistencia, 
                        Inductancia, 
                        Capacitancia, 
                        lts_path,
                        tSim = tiempoSimulacion,
                        toff_sw = tiempoSwitch,
                        ton_mos = tiempoMos)
    
    dibujar(resultado)
    